;*************************************************************************************************************************
;*************************************************************************************************************************
;      						RUTINES PEL LA COMUNICACI ENTRE EL PIC16F886 I L'ACCELERMETRE BMA180

;					Autor:		Joel Muoz Puigdevall
;					Projecte:	Disseny i desenvolupament d'un sistema de posicionament inercial
;					Data: 		20/02/2012


;Conjunt de rutines per la comunicaci entre el PIC16F886 i l'accelermetre BMA180.

;*************************************************************************************************************************
;*************************************************************************************************************************
;Inicialitzaci de la connexi entre PIC-BMA180:
BMA180_I	bsf		STATUS,RP0			;Banc 1
			bsf		TRISA,4				;RA4 entrada
			bcf		STATUS,RP0			;Banc 0

			;Temps de power-on (2.5ms):
			movlw	.130
			movwf	Retard_1
			call	Retard

			;Configurem registres pel mode de funcionament:
			call 	I2C_Start			;Inici comunicaci
			movlw	0x80
			call	I2C_Enviar_Byte		;Enviar l'adrea de l'accelermetre, mode escritura
			movlw	0x0D
			call	I2C_Enviar_Byte		;Enviar adrea de memria ctrl_reg0
			movlw	0x10
			call	I2C_Enviar_Byte		;ee_w = 1, ara puc escriure sobre l'imatge
			call 	I2C_RStart			;Reinici comunicaci
			movlw	0x80
			call	I2C_Enviar_Byte		;Enviar l'adrea de l'accelermetre, mode escritura
			movlw	0x21
			call	I2C_Enviar_Byte		;Enviar adrea de memria ctrl_reg3
			movlw	0x04
			call	I2C_Enviar_Byte		;inhabilitar interrupcions	
			call 	I2C_RStart			;Reinici comunicaci
			movlw	0x80
			call	I2C_Enviar_Byte		;Enviar l'adrea de l'accelermetre, mode escritura
			movlw	0x20
			call	I2C_Enviar_Byte		;Enviar adrea de memria bw_tcs
			movlw	0x08
			call	I2C_Enviar_Byte		;bw = 0000, ampla de banda de 10Hz
			call 	I2C_RStart			;Reinici comunicaci
			movlw	0x80
			call	I2C_Enviar_Byte		;Enviar l'adrea de l'accelermetre, mode escritura
			movlw	0x35
			call	I2C_Enviar_Byte		;Enviar adrea de memria offset_lsb1
			movlw	0x55
			call	I2C_Enviar_Byte		;smp_skip = 1, adaptar la interrupci a l'ample de banda escollit
			call 	I2C_RStart			;Reinici comunicaci
			movlw	0x80
			call	I2C_Enviar_Byte		;Enviar l'adrea de l'accelermetre, mode escritura
			movlw	0x21
			call	I2C_Enviar_Byte		;Enviar adrea de memria ctrl_reg0
			movlw	0x02
			call	I2C_Enviar_Byte		;new_data_int = 1, interrupci de nova dada activada
			call 	I2C_RStart			;Reinici comunicaci
			movlw	0x80
			call	I2C_Enviar_Byte		;Enviar l'adrea de l'accelermetre, mode escritura
			movlw	0x33
			call	I2C_Enviar_Byte		;Enviar adrea de memria gain_y
			movlw	0x9B
			call	I2C_Enviar_Byte		;shadow_dis = 1, mentres llegim LSB, bloquejem MSB
			call	I2C_Stop			;Fi comunicaci

			return

;*************************************************************************************************************************
;Lectura d'acceleraci:
BMA180_R	call 	I2C_Start			;Inici comunicaci
			movlw	0x80
			call	I2C_Enviar_Byte		;Enviar l'adrea de l'accelermetre, mode escritura
			movf	Memoria,W			;Eix a llegir
			call	I2C_Enviar_Byte		;Enviar adrea de memria acc_x_lsb
			call 	I2C_RStart			;Reinici comunicaci
			movlw	0x81
			call	I2C_Enviar_Byte		;Enviar l'adrea de l'accelermetre, mode lectura
			call	I2C_Llegir_Byte		;Llegir dades, LSB
			call	I2C_nACK			;Enviar nACK (he rebut la dada)
			incf	FSR					;Incrementem punter
			movwf	INDF				;Guardo el 1r byte
			decf	FSR					;Decrementem punter
			call	I2C_Llegir_Byte		;Llegir dades, MSB
			call	I2C_ACK				;Enviar ACK,fi de trasmissi
			movwf	INDF				;llegeixo 2n byte
			incf	FSR					;Incrementem punter
			incf	FSR					;Incrementem punter
			call	I2C_Stop			;Fi comunicaci
			return

;*************************************************************************************************************************