;*************************************************************************************************************************
;*************************************************************************************************************************
;      						RUTINES PEL LA COMUNICACI ENTRE EL PIC16F886 I LA BRIXOLA HMC6352

;					Autor:		Joel Muoz Puigdevall
;					Projecte:	Disseny i desenvolupament d'un sistema de posicionament inercial
;					Data: 		20/02/2012


;Conjunt de rutines per la comunicaci entre el PIC16F886 i la brixola HMC6352.

;*************************************************************************************************************************
;*************************************************************************************************************************
;Inicialitzaci de la connexi entre PIC-HMC6352:
HMC6352_I	bsf		STATUS,RP0
			bsf		STATUS,RP1			;Banc 3
			bcf		ANSELH,ANS8			;RB2 digital
			bcf		ANSELH,ANS9			;RB3 digital
			bcf		STATUS,RP1			;Banc 1
			bsf		TRISB,2				;RB2 entrada
			bcf		TRISB,3				;RB3 sortida
			bcf		STATUS,RP0			;Banc 0
			bcf		PORTB,3				;Estat inicial, RB3=0

			;Temps de power-on (50ms):
			movlw	.250
			movwf	Retard_1
			call	Retard

			call 	I2C_Start			;Inici comunicaci
			movlw	0x42
			call	I2C_Enviar_Byte		;Enviar l'adrea de la brixola, mode escritura
			movlw	0x47
			call	I2C_Enviar_Byte		;Enviar comanda: escriure en un registre de la RAM
			movlw	0x74
			call	I2C_Enviar_Byte		;Registre a escriure, 74h (mode d'operaci)
			movlw	0x72
			call	I2C_Enviar_Byte		;Continuous Mode
			call	I2C_Stop			;Fi comunicaci

			movlw	.100
			movwf	Retard_1
			call	Retard				;Temporitza (temps d'execuci de la brixola)			

			;Temps per la primera lectura (50ms):
			movlw	.250
			movwf	Retard_1
			call	Retard

			return

;*************************************************************************************************************************
;Lectura dels graus respecte el nord:
HMC6352_R	call 	I2C_Start			;Inici comunicaci
			movlw	0x43
			call	I2C_Enviar_Byte		;Enviar l'adrea de la brixola, mode lectura
			call	I2C_Llegir_Byte		;Llegir dades, la brixola envia el 1r byte
			call	I2C_nACK			;Enviar nACK (he rebut la dada)
			movwf	INDF				;Guardo el 1r byte
			incf	FSR					;Incrementem punter
			call	I2C_Llegir_Byte		;Llegir dades, la brixola envia el 2n byte
			call	I2C_ACK				;Enviar ACK,fi de transmissi
			movwf	INDF				;llegeixo 2n byte
			incf	FSR					;Incrementem punter
			call	I2C_Stop			;Fi comunicaci
			return

;*************************************************************************************************************************
;Calibraci de la brixola (els camps magntics distorcionen als resultats de mesura de la brixola si
;hi estan a prop, amb la calibraci es tenen en compte els camps magntics dels voltants i es compensen):
HMC6352_C	clrf	CCP1CON				;Desactivem em mdul CCP1

			;ENTRAR EN MODE CALIBRACI:
			call 	I2C_Start			;Inici comunicaci
			movlw	0x42
			call	I2C_Enviar_Byte		;Enviar l'adrea de la brixola, mode escritura
			movlw	0x43
			call	I2C_Enviar_Byte		;Enviar comanda: entrar en mode calibraci
			call	I2C_Stop			;Fi comunicaci

			movlw	.250
			movwf	Retard_1
			call	Retard				;Temporitza uns 200us (temps d'execuci de la brixola)

			call	LED_interm			;Intermitncia del led (ens indica que estem calibrant)

			;SORTIR DE MODE CALIBRACI
			call 	I2C_Start			;Inici comunicaci
			movlw	0x42
			call	I2C_Enviar_Byte		;Enviar l'adrea de la brixola, mode escritura
			movlw	0x45
			call	I2C_Enviar_Byte		;Enviar comanda: sortir de mode calibraci
			call	I2C_Stop			;Fi comunicaci

			movlw	.75
			movwf	Retard_1
			call	Retard				;Temporitza uns 15000us (temps d'execuci de la brixola)
			call	CCP1_I				;Inicialitzar el mdul del CCP1

			return

;*************************************************************************************************************************
;Intermitncia del LED de recopilaci de dades per corregir els errors que provquen els camps magnetics:
LED_interm	bsf		PORTB,3				;Activem led, recopilaci de dades
			movlw	.50
			movwf	Memoria
LED_wait1	movlw	.25
			movwf	Retard_1
			call	Retard
			decfsz	Memoria
			goto	LED_wait1			;Intermitncia de 250ms

			bcf		PORTB,3				;Desactivem led, recopilaci de dades
			movlw	.50
			movwf	Memoria
LED_wait2	movlw	.25
			movwf	Retard_1
			call	Retard
			decfsz	Memoria
			goto	LED_wait2			;Intermitncia de 250ms
			
			btfsc	PORTB,2				;Dades recopilades?
			goto	LED_interm			;No, ESTEM RECOPILANT DADES PER COMPENSAR ELS CAMPS MAGNTICS...

			return

;*************************************************************************************************************************