;*************************************************************************************************************************
;*************************************************************************************************************************
;      						RUTINES PEL PROTOCOL DE COMUNICACI SRIE I2C

;					Autor:		Joel Muoz Puigdevall
;					Projecte:	Disseny i desenvolupament d'un sistema de posicionament inercial
;					Data: 		20/02/2012


;Conjunt de rutines per al protocol de comunicaci srie I2C mitjanant un microcontrolador PIC16F886.

;*************************************************************************************************************************
;*************************************************************************************************************************
;I2C_I: Ajusta el mdul MSSP al mode Master amb freqncia de treball = Fosc/(4*(SPADD+1)) = 100KHz

I2C_I			bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				bsf		STATUS,RP0		;Banc 1
				bsf		TRISC,3
				bsf		TRISC,4			;RC3 i RC4 com a sortides
				movlw	b'10000000'
				movwf	SSPSTAT			;Velocidad estndar en I2C
				movlw	.9
				movwf	SSPADD			;Valor a SSPADD per una velocitat de 100KHz
				bcf		STATUS,RP0		;Banc 0
				movlw	b'00101000'
				movwf	SSPCON			;Mdul MSSP a ON, mode Master y Fscl = Fosc/(4*(SPADD+1)) = 100KHz
				return

;*************************************************************************************************************************
;I2C_Start: Envia la seqncia d'inici

I2C_Start		bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				bsf		STATUS,RP0		;Banc 1
				bsf		SSPCON2,SEN		;Activa seqncia d'inici
				bcf		STATUS,RP0		;Banc 0
I2C_Start_wait	btfss	PIR1,SSPIF		;Seqncia d'inici acabada?
				goto	I2C_Start_wait	;No, esperar
				bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP		
				return

;*************************************************************************************************************************
;I2C_RStart: Envia la seqncia de re-inici

I2C_RStart		bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				bsf		STATUS,RP0		;Banc 1
				bsf		SSPCON2,RSEN	;Activa seqncia de re-inici
				bcf		STATUS,RP0		;Banc 0
I2C_RStart_wait	btfss	PIR1,SSPIF		;Seqncia de re-inici acabada?
				goto	I2C_RStart_wait	;No, esperar
				bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP		
				return

;*************************************************************************************************************************
;I2C_Stop: Envia la seqncia de stop

I2C_Stop		bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				bsf		STATUS,RP0		;Banc 1
				bsf		SSPCON2,PEN		;Activa seqncia de stop
				bcf		STATUS,RP0		;Banc 0
I2C_Stop_wait	btfss	PIR1,SSPIF		;Seqncia de stop acabada?
				goto	I2C_Stop_wait	;No, esperar
				bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				return

;*************************************************************************************************************************
;I2C_Enviar_Byte: Transmet el byte del W per I2C. La rutina finalitza quan es rep /ACK

I2C_Enviar_Byte	bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				movwf	SSPBUF			;Byte a transmetre al buffer de sortida
				bsf		STATUS,RP0		;Banc 1
I2C_RW			btfsc	SSPSTAT,R_W		;Esperar que acabi la transmissi
				goto	I2C_RW
				bcf		STATUS,RP0		;Banc 0
I2C_Enviar_wait	btfss	PIR1,SSPIF		;Rebut el bit /ACK?
				goto	I2C_Enviar_wait	;No, esperar
				bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				return

;*************************************************************************************************************************
;I2C_Llegir_Byte: Llegeix el byte procedent del dispositio I2C i el retorna a W.


I2C_Llegir_Byte	bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				bsf		STATUS,RP0		;Banc 1
				bsf		SSPCON2,RCEN	;Activa el mode de recepci I2C
				bcf		STATUS,RP0		;Banc 0
I2C_Llegir_wait	btfss	PIR1,SSPIF		;Rebuts els 8 bits?
				goto	I2C_Llegir_wait	;No, esperar
				movf	SSPBUF,W		;Llegeix el byte rebut
				bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				return

;*************************************************************************************************************************
;I2C_nACK: Envia el bit ACK=0, byte rebut

I2C_nACK		bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				bsf		STATUS,RP0		;Banc 1		
				bcf		SSPCON2,ACKDT	;Posa el bit ACKDT a "0"
				bsf		SSPCON2,ACKEN	;Aciva la seqncia de generaci del bit ACK
				bcf		STATUS,RP0		;Banc 0
I2C_nACK_wait	btfss	PIR1,SSPIF		;Seqncia ACK finalitzada?
				goto	I2C_nACK_wait	;No, esperar
				bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				return

;*************************************************************************************************************************
;I2C_ACK: Envia el bit ACK=1, final de recepci

I2C_ACK			bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				bsf		STATUS,RP0		;Banc 1		
				bsf		SSPCON2,ACKDT	;Posa el bit ACKDT a "1"
				bsf		SSPCON2,ACKEN	;Aciva la seqncia de generaci del bit ACK
				bcf		STATUS,RP0		;Banc 0
I2C_ACK_wait	btfss	PIR1,SSPIF		;Seqncia ACK finalitzada?
				goto	I2C_ACK_wait	;No, esperar
				bcf		PIR1,SSPIF		;Restaura el flag del mdul MSSP
				return

;*************************************************************************************************************************