;*************************************************************************************************************************
;*************************************************************************************************************************
;      						RUTINES PEL LA COMUNICACI ENTRE EL PIC16F886 I EL GIROSCPIC IXZ500

;					Autor:		Joel Muoz Puigdevall
;					Projecte:	Disseny i desenvolupament d'un sistema de posicionament inercial
;					Data: 		20/02/2012


;Conjunt de rutines per la comunicaci entre el PIC16F886 i el giroscpic IXZ500.

;*************************************************************************************************************************
;*************************************************************************************************************************
;Variables de memria (rutina de temporitzaci):
			CBLOCK	Retard_var						
					Retard_1
					Retard_2
			ENDC

;*************************************************************************************************************************
;Petit retard, til per evitar utilitzar el TIMER del PIC i deixar-lo lliure per a altres aplicacions:
Retard		movlw	.50
			movwf	Retard_2
Retard_re	clrwdt
			decfsz	Retard_2,F
			goto	Retard_re
			decfsz	Retard_1,F	;Bucle de 200us que es repeteix Retard_1 vegades
			goto	Retard
			return

;*************************************************************************************************************************
;Inicialitzaci de la connexi entre PIC-IXZ500:
IXZ500_I	bsf		STATUS,RP0
			bsf		STATUS,RP1			;Banc 3
			bsf		ANSEL,ANS0			;RA0 analgic (lectura X OUT)
			bsf		ANSEL,ANS1			;RA1 analgic (lectura Z OUT)
			bcf		ANSEL,ANS2			;RA2 digital  (AutoZero)
			bcf		STATUS,RP1			;Banc 1
			bcf		TRISA,2				;RA2 sortida (AutoZero)
			bcf		STATUS,RP0			;Banc 0
			bcf		PORTA,2				;Estat inicial, RA0=0

			;AutoZero, calibrar l'acceleraci nulla a Vref = 1.35V:
			movlw	.100
			movwf	Retard_1
			call	Retard				;Esperem els 20ms d'Start up del giroscpic
			bsf		PORTA,2				;Activem AZ
			movlw	.1
			movwf	Retard_1
			call	Retard				;Duraci d'uns 200us suficients pel puls AZ
			bcf		PORTA,2				;Desactivem AZ
			movlw	.50
			movwf	Retard_1
			call	Retard				;Temporitza uns 10mS per assegurar l'Auto Zero

			;Inicialitzaci del mdul ADC:
			bsf		STATUS,RP0			;Banc 1
			movlw	b'10010000'
			movwf	ADCON1				;Alineaci dreta, Vref= 0-3.3V
			bcf		STATUS,RP0			;Banc 0

			;Temps de power-on (50ms):
			movlw	.50
			movwf	Memoria
IXZ500_PO	movlw	.5
			movwf	Retard_1
			call	Retard
			decfsz	Memoria
			goto	IXZ500_PO

			return

;*************************************************************************************************************************
;Lectura de velocitat angular:
IXZ500_R	movf	Memoria,W		;Eix a llegir
			movwf	ADCON0			;ADC en On, seleciona canal analgic a llegir, clock=Fosc/8

			;Esperar un temps d'adquisici (11us, suficients per carregar Chold):
			movlw	.3
			movwf	Retard_1
CHOLD		decfsz	Retard_1
			goto	CHOLD

			;Inici conversi:
			bcf		PIR1,ADIF
			bsf		ADCON0,GO		;Inicia la conversi
IXZ500_Wait	btfss	PIR1,ADIF		;conversi finalitzada?
			goto	IXZ500_Wait		;No, espera.
			bcf		ADCON0,ADON		;Si, apaguem ADC
			bcf		PIR1,ADIF
			movlw	b'00000011'
			andwf	ADRESH,W
			movwf	INDF			;Guardo el 1r byte (MSB)
			incf	FSR				;Incrementem punter
			bsf		STATUS,RP0		;Banc 1
			movf	ADRESL,W
			bcf		STATUS,RP0		;Banc 0
			movwf	INDF			;Guardo el 1r byte (LSB)
			incf	FSR				;Incrementem punter

			return

;*************************************************************************************************************************
