;*************************************************************************************************************************
;*************************************************************************************************************************

;            COMUNICACI DEL PIC 16F886 AMB L'ACCELERMETRE BMA180 PER MITJ DEL PROTOCOL DE COMUNICACI I2C

;*************************************************************************************************************************
;*************************************************************************************************************************

			List	p=16F886			;Tipus de microcontrolador
			include	"P16F886.inc"		;Definicions de registres interns

			__config	_CONFIG1, _LVP_OFF&_PWRTE_ON&_WDT_OFF&_EC_OSC&_FCMEN_OFF	;Registre 1 de configuraci
			__config	_CONFIG2, _WRT_OFF&_BOR40V									;Registre 2 de configuraci

Memoria		equ		0x20				;Variable de memria
Comptador	equ		0x21				;Variable de temporitzaci
Retard_var	equ		0x22				;Variables de mmria (Rutina de temporitzaci)

			org		0x00				;Vector RESET	
			goto	Inici
			org		0x05				;Vector CODI PROGRAMA

			include "BMA180.inc"		;Rutines per l'accelermetre BMA180
			include "IXZ500.inc"		;Rutines pel giroscpic IXZ500
			include "HMC6352.inc"		;Rutines per la brixola HMC6352
			include "I2C.inc"			;Rutines per la comunicaci I2C
			include "UART.inc"			;Rutines per la comunicaci per UART

;*************************************************************************************************************************
;FUNCIONS
;*************************************************************************************************************************
;Inicialitzaci del mdul CCP1 per enviar dades al PC cada 50ms:
CCP1_I		clrf	T1CON
			clrf	CCP1CON
			bsf		STATUS,RP0
			bcf		PIE1,TMR1IE
			bcf		PIE1,CCP1IE
			bcf		STATUS,RP0
			clrf	PIR1
			movlw	b'00110001'
			movwf	T1CON				;Seleccionar rellotge intern y preescaler de 8
			movlw	0x0B
			movwf	CCP1CON				;Seleccionar mode comparador, amb reset al Timer1
			movlw	0x18
			movwf	CCPR1H
			movlw	0x6A
			movwf	CCPR1L				;CCPR1 = 50ms
			bsf		T1CON,TMR1ON
			return

;*************************************************************************************************************************
;Inicialitzaci del LED que indica que la placa ja est inicialitzada i preparada per enviar valors vlids de trajectria:
LED_I		bsf		STATUS,RP0
			bsf		STATUS,RP1			;Banc 3
			bcf		ANSELH,ANS12		;RB0 digital
			bcf		ANSELH,ANS10		;RB1 digital
			bcf		STATUS,RP1			;Banc 1
			bcf		TRISB,0				;RB0 sortida
			bcf		TRISB,1				;RB1 sortida
			bcf		STATUS,RP0			;Banc 0
			bcf		PORTB,0
			bsf		PORTB,1				;Estat inicial, RB0=0(led verd) i RB1=1(led vermell)
			return

;*************************************************************************************************************************
;Esperem 1 segon tant per a l'estabilitat de la plataforma com l'alimentaci dels sensors:
ESTABILITAT	movlw	.20
			movwf	Memoria
ESTAB_W		movlw	.250
			movwf	Retard_1
			call	Retard
			decfsz	Memoria
			goto	ESTAB_W
			return

;*************************************************************************************************************************
;PROGRAMA PRINCIPAL
;*************************************************************************************************************************
Inici		call	UART_I				;Inicialitzar comunicaci per UART
			call	I2C_I				;Inicialitzar bus I2C
			call	BMA180_I			;Inicialitzar connexi entre PIC-BMA180
			call	IXZ500_I			;Inicialitzar connexi entre PIC-IXZ500
			call	HMC6352_I			;Inicialitzem connexi entre PIC-HMC6352
			call	LED_I				;Inicialitzar el LED indicador
			call	ESTABILITAT			;Esperem un temps prudencial d'estabilitat
			movlw	.100
			movwf	Comptador			;100x50ms=5s (temps d'espera per la calibraci inicial de la placa amb el MATLAB)

			;Configuraci direccionament indirecte de dades:
			bcf		STATUS,IRP			;Bancs 0 i 1
			call	CCP1_I				;Inicialitzar el mdul del CCP1
Loop		movlw	0x30
			movwf	FSR					;Direcci a partir de la qual s'aniran guardan totes les dades (0x30)

			;Lectura d'acceleraci:
BMA180_W	btfss	PORTA,4				;Dades d'acceleraci disponibles?
			goto	BMA180_W			;No, esperar
			movlw	0x02
			movwf	Memoria				;Eix X
			call	BMA180_R			;Llegir dades d'acceleraci
			movlw	0x04
			movwf	Memoria				;Eix Y
			call	BMA180_R			;Llegir dades d'acceleraci
			movlw	0x06
			movwf	Memoria				;Eix Z
			call	BMA180_R			;Llegir dades d'acceleraci

			;Lectura de velocitat angular:
			movlw	b'01000001'
			movwf	Memoria				;Eix X (XOUT)
			call	IXZ500_R			;Llegir dades de velocitat angular
			movlw	b'01000101'
			movwf	Memoria				;Eix Z (ZOUT)
			call	IXZ500_R			;Llegir dades de velocitat angular

			;Lectura de la brixola:
			call	HMC6352_R			;Llegir dades de la brixola

			;Transmissi de dades al PC per UART:
			movlw	0x30
			movwf	FSR					;Direcci a partir de la qual s'aniran envian totes les dades (0x30)
			movlw	.12
			movwf	Memoria				;Enviar 12 dades (6 accelermetre + 4 giroscpic + 2 brixola)
			call	UART_T

			;Bucle d'error, la lectura de les dedes dels sensors han tardat ms de 50ms:
			bsf		PORTB,1
			bcf		PORTB,0
bucle		btfsc	PIR1,CCP1IF			;Han passat 50ms o ms?
			goto	bucle				;Si, error! Reiniciar el PIC.

			;Comprovaci dels 5s per a la calibraci de les dades per MATLAB:
			movf	Comptador,W
			sublw	.0
			btfss	STATUS,C			;Comptador = 0?
			decf	Comptador			;No, decrementa
			btfss	STATUS,C			;Comptador = 0?
			goto	Compara				;No, seguim amb el led vermell activat
			bcf		PORTB,1				;Si, desactivem led vermell i activem led verd. Calibraci concluda! Tot
			bsf		PORTB,0				;preparat per enviar dades de la trajectria

			;Espera que hagin passat 50ms. Si s aix, es resseteja el CCP1 i es repeteix el cicle: 
Compara		btfss	PIR1,CCP1IF			;Han passat justament 50ms?
			goto	Compara				;No, espera.
			bcf		PIR1,CCP1IF			;Si, re-set a CCP1 i repetim cicle
			btfsc	PORTB,2				;Calibrar brixola?
			call	HMC6352_C			;Si
			goto	Loop

			end							;Fi del programa

;*************************************************************************************************************************